# coding=utf-8
# django_telegram_bot/apps.py
from django.apps import AppConfig
from django.conf import settings
import importlib
import telegram

class DjangoTelegramBot(AppConfig):

    name = 'django_telegrambot'
    verbose_name = 'Django TelegramBot'
    ready_run = False
    dispatcher = None
    bot_tokens = []
    bot_usernames = []
    dispatchers = []
    bots = []
    
    @classmethod
    def getDispatcher(cls, id = None, safe=True):
        if id == None:
            return dispatchers[0]
        else:
            try:
                index = cls.bot_tokens.index(id)
            except ValueError:
                if not safe : return None
                try:
                    index = cls.bot_usernames.index(id)
                except ValueError:
                    return None
            return cls.dispatchers[index]
            
    @classmethod
    def getBot(cls, id = None, safe = True):
        if id == None:
            return bots[0]
        else:
            try:
                index = cls.bot_tokens.index(id)
            except ValueError:
                if not safe : return None
                try:
                    index = cls.bot_usernames.index(id)
                except ValueError:
                    return None
            return cls.bots[index]
        
    
    def ready(self):
        if DjangoTelegramBot.ready_run: return
        DjangoTelegramBot.ready_run = True
        
        if not hasattr(settings, 'TELEGRAM_BOT_TOKENS'): return
        tokens = settings.TELEGRAM_BOT_TOKENS
        
        if not hasattr(settings, 'TELEGRAM_WEBHOOK_SITE'): return
        webhook_site = settings.TELEGRAM_WEBHOOK_SITE
        
        if not hasattr(settings, 'TELEGRAM_WEBHOOK_BASE'): return
        webhook_base = settings.TELEGRAM_WEBHOOK_BASE
        
        if hasattr(settings, 'TELEGRAM_WEBHOOK_CERTIFICATE'):
            CERT = settings.TELEGRAM_WEBHOOK_CERTIFICATE
        
        for index, token in enumerate(tokens):
            
            bot = telegram.Bot(token=token)
            hookurl = '{}{}/{}/'.format(webhook_site,webhook_base, token)
            if hasattr(settings, 'TELEGRAM_WEBHOOK_CERTIFICATE'):
                setted = bot.setWebhook(hookurl, certificate=open(CERT,'rb'))
            else:
                setted = bot.setWebhook(hookurl, certificate=None)
                
            print ('Telegram Bot <{}> setting webhook [ {} ] : {}'.format(bot.username,hookurl,setted))
            
            DjangoTelegramBot.dispatchers.append(telegram.Dispatcher(bot, None))
            DjangoTelegramBot.bots.append(bot)
            DjangoTelegramBot.bot_tokens.append(bot.token)
            DjangoTelegramBot.bot_usernames.append(bot.username)
            
            #per compatibilità salvo il primo bot nella proprietà DjangoTelegramBot.dispatcher
            if index==0:
                DjangoTelegramBot.dispatcher = DjangoTelegramBot.dispatchers[0]
                
            
        def module_exists(module_name, method_name, execute):
            try:
                #m = __import__(module_name).telegrambot
                m = importlib.import_module(module_name)
                if execute and hasattr(m, method_name):
                    print ('Run {}.main()'.format(module_name))
                    getattr(m,method_name)()
                else:
                    print ('Run {}'.format(module_name))
                        
            except ImportError:
                return False
            
            return True
        
        # import telegram bot handlers for all INSTALLED_APPS
        for app in settings.INSTALLED_APPS:
            module_name = '{}.telegrambot'.format( app )
            if module_exists(module_name, 'main', True):
                print ('Loaded {}'.format( module_name))
                
            
        #import telegrambot
        #telegrambot.main() # in alternativa si potrebbe imporre l'uso di un main in telegrambot.py
        


# -*- coding: utf-8 -*-
# Example code for telegrambot.py module
from django_telegrambot.apps import DjangoTelegramBot

import logging
logger = logging.getLogger(__name__)


#in questa parte del file definire il funzionamento del bot aggiungendo gli handler
def start(bot, update):
    bot.sendMessage(update.message.chat_id, text='Hi!')

def help(bot, update):
    bot.sendMessage(update.message.chat_id, text='Help!')

def echo(bot, update):
    logger.info('ECHO: %s'%update.message.text)
    bot.sendMessage(update.message.chat_id, text=update.message.text)

def error(bot, update, error):
    logger.warn('Update "%s" caused error "%s"' % (update, error))
    

def main():
    logger.info("Loading handlers for telegram bot")
    
    # Default dispatcher (this is related to the first bot in settings.TELEGRAM_BOT_TOKENS)
    dp = DjangoTelegramBot.dispatcher
    # To get Dispatcher related to a specific bot
    # dp = DjangoTelegramBot.getDispatcher('BOT_n_token')     #get by bot token
    # dp = DjangoTelegramBot.getDispatcher('BOT_n_username')  #get by bot username
    
    # on different commands - answer in Telegram
    dp.addTelegramCommandHandler("start", start)
    dp.addTelegramCommandHandler("help", help)

    # on noncommand i.e message - echo the message on Telegram
    dp.addTelegramMessageHandler(echo)

    # log all errors
    dp.addErrorHandler(error)




# coding=utf-8
from django.shortcuts import render
from django.http import JsonResponse
from django_telegrambot.apps import DjangoTelegramBot
from django.views.decorators.csrf import csrf_exempt
import json
import telegram
# import the logging library
import logging

# Get an instance of a logger
logger = logging.getLogger(__name__)


# Create your views here.

@csrf_exempt
def webhook (request, bot_token):
    
    #verifico la validità del token
    if DjangoTelegramBot.getBot(bot_token, safe=False) is None:
        return JsonResponse({})
    
    try:
        data = json.loads(request.body.decode("utf-8"))
        logger.info(data)
    except:
        logger.info('Telegram bot receive invalid request' )
        return JsonResponse({})

    dispatcher = DjangoTelegramBot.getDispatcher(bot_token, safe=False)
    if dispatcher is None:
        return JsonResponse({})
        
    try:
        update = telegram.Update.de_json(data)
        dispatcher.processUpdate(update)
        logger.debug('Processed Update: {} with context {}'.format(update, context))
    # Dispatch any errors
    except TelegramError as te:
        logger.warn("Error was raised while processing Update.")
        dispatcher.dispatchError(update, te)

    # All other errors should not stop the thread, just print them
    except:
        logger.error("An uncaught error was raised while processing an update")
    
    finally:
        return JsonResponse({})
